// Ornek MPI programi
// Iki tane blok dagitilimis rastgele vektorun 
// ic carpimi hesaplanir
// Eray Ozkural

#include <algorithm>
#include <iostream>
#include <cstdio>
#include <cstdlib>
using namespace std;
#include <mpi.h>

// iki vektorun ic carpimi
template<typename T>
T dot_product(T* a, T* b, int len) {
  T p = 0;
  for (int i=0; i<len; i++)
    p +=  a[i] * b[i];
  return p;
}

// rastgele sayi dizisi olustur
void rand_array(float* a, int len, float range) 
{
  srandom((unsigned)time(NULL));
  for (int i=0; i<len; i++) {
    a[i] = range / (int(random()) % 0x7fffffff);
  }
}

template<typename T>
void print_vector(T* a, int len)
{
  for (int i=0; i<len; i++)
    cout << a[i] << " ";
  cout << endl;
}

int main(int argc, char **argv)
{
  MPI::Init(argc,argv);
  MPI::Intracomm comm = MPI_COMM_WORLD;
  int numprocs = comm.Get_size();
  int id = comm.Get_rank();

  float a[10];
  rand_array(a, 10, 1.0); // 10 random numbers in (0,1)
  float b[10];
  rand_array(b, 10, 1.0); // 10 random numbers in (0,1)
  float localdotp = dot_product(a, b, 10);
  float globaldotp;
  comm.Reduce(&localdotp, &globaldotp, 1, MPI::FLOAT, MPI::SUM, 0);
  if (id==0)
    cout << "a.b = " << globaldotp << endl;

  MPI::Finalize();
  return 0;
}
