// Ornek MPI programi
// islemci 0, islemci 1'e 10 tane tamsayi gonderir
// Eray Ozkural

#include <algorithm>
#include <iostream>
#include <cstdio>
#include <cstdlib>
using namespace std;
#include <mpi.h>

// rastgele tamsayi dizisi olustur
void rand_array(int* a, int len, int range) 
{
  srandom((unsigned)time(NULL));
  for (int i=0; i<len; i++) {
    a[i] = int(random()) % range;
  }
}

// iki tamsayi vektorunun ic carpimi
int dot_product(int* a, int* b, int len) {
  int p = 0;
  for (int i=0; i<len; i++)
    p +=  a[i] * b[i];
  return p;
}

int main(int argc, char **argv)
{
  MPI::Init(argc,argv);
  MPI::Intracomm comm = MPI_COMM_WORLD;
  int numprocs = comm.Get_size();
  int id = comm.Get_rank();
  if (numprocs!=2) {
    cout << "Bu program sadece iki islemciyle calisir." << endl;
    return 1; // error
  }

  int a[10];
  rand_array(a, 10, 100); // 10 random ints in 0..99
  if (id==0)
    comm.Send(a, 10, MPI::INT, 1, 13);
  else {
    int buf[10];
    comm.Recv(buf, 10, MPI::INT, 0, 13);
    int dot_prod = dot_product(a, buf, 10);
    cout << "Iki tamsayi vektorun carpimi = " << dot_prod << endl;
  }

  MPI::Finalize();
  return 0;
}
